@@ -11,6 +11,7 @@ import json
 import yaml
 import shutil
 import threading
+import dotenv
 from datetime import datetime
 from github import Github
 from botocore.exceptions import WaiterError
@@ -62,6 +63,7 @@ class EC2Manager:
             )
             self._aws_default_region = self._config.get('aws_region')
             self._type = self._config.get('type')
+            self._category = self._config.get('category')
             self._vpc_name = self._config.get('vpc_name')
             self._public_subnet_cidr = self._config.get('public_subnet_cidr')
             self._expose_docker_daemons = int(self._config.get('expose_docker_daemons', 0))
@@ -98,7 +100,8 @@ class EC2Manager:
                     'environment variable.'
                 )
             self._aws_backend_bucket = os.environ.get('AWS_BACKEND_BUCKET', f'{account_number}-terraform-state')
-            self._aws_backend_key = f'{self._type}.tfstate'
+            self._aws_backend_folder = '/'.join([i for i in [self._category, self._type] if i])
+            self._aws_backend_key = '/'.join([self._aws_backend_folder, 'state.tfstate'])
 
     def get_config(self):
         """
@@ -113,7 +116,7 @@ class EC2Manager:
         return {}
 
     @staticmethod
-    def _get_encoded_envs(encoded_env_data):
+    def _variables_from_env_file(encoded_env_data):
         """
         Gets environment variables from the given encoded string.
 
@@ -190,6 +193,22 @@ class EC2Manager:
             commit_time = repo.get_commit(self._commit).last_modified
             return datetime.strptime(commit_time, '%a, %d %b %Y %H:%M:%S %Z').astimezone(pytz.UTC)
 
+    def _get_secret_envs(self, name):
+        """
+        Get the secret envs from s3.
+        """
+        envs = {}
+        for secret_file_name in self._config['instances'][name].get('secret_files', []):
+            secret_file_name = secret_file_name.strip('/').replace('\\', '/')
+            _, ext = os.path.splitext(secret_file_name)
+            if ext == '.env':
+                data = self._s3_client.get_object(
+                    Bucket=self._aws_backend_bucket,
+                    Key=f'{self._aws_backend_folder}/{secret_file_name}'
+                )
+                envs.update(dotenv.dotenv_values(stream=data['Body'], encoding='utf-8'))
+        return envs
+
     def _get_instance_data(self):
         """
         Gets all the instance's environment variables.
@@ -217,7 +236,7 @@ class EC2Manager:
                 'ENV_FILE_NAME': os.path.basename(instances[name].get('encoded_env_file', {}).get('file_name', '')),
                 'GITHUB_USERNAME': self.github_user,
                 **instances[name].get('envs', {}),
-                **self._get_encoded_envs(instances[name].get('encoded_env_file', {}))
+                **self._get_secret_envs(name)
             }
         } for name in self._instance_repo_updates}
 
@@ -357,7 +376,7 @@ class EC2Manager:
         :param callable callable_instance: A callable.
         :param Any items: A list of items to call.
         :param list extra_args: A list of extra arguments.
-        :param bool join: Whether or not to join the threads.
+        :param bool join: Whether to join the threads.
         :return list[Thread]: A list of threads.
         """
         if not extra_args:
@@ -603,6 +622,8 @@ class EC2Manager:
         """
         Updates a single instance repo.
         """
+        self.upload_file_secret_files(name)
+
         if self.repo not in self.run_command(name, ['ls'], print_output=False):
             self._logger.info(f'{name} cloning {self.repo}')
             self.run_command(name, [f'git clone {self._repo_url}'])
@@ -611,7 +632,8 @@ class EC2Manager:
         commands = [
             f'cd ./{self.repo}',
             'git fetch',
-            'git pull'
+            'git pull',
+            *self._get_file_secret_download_commands(name)
         ]
 
         # set the repo folder permissions for the default user
@@ -661,6 +683,34 @@ class EC2Manager:
             data['commands']['start'],
         ])
 
+    def _get_file_secret_download_commands(self, name):
+        # sync down secret files
+        commands = []
+        for secret_file_name in self._config['instances'][name].get('secret_files', []):
+            secret_file_name = secret_file_name.strip('/').replace('\\', '/')
+            commands.append(
+                f'aws s3 cp '
+                f's3://{self._aws_backend_bucket}/{self._aws_backend_folder}/{secret_file_name} '
+                f'{secret_file_name}'
+            )
+        return commands
+
+    def upload_file_secret_files(self, name):
+        """
+        Uploads the secret files listed in the config.
+        """
+        for secret_file_name in self._config['instances'][name].get('secret_files', []):
+            secret_file_name = secret_file_name.strip('/').replace('\\', '/')
+            secret_file_path = os.path.join(os.getcwd(), 'compose', secret_file_name)
+            if os.path.exists(secret_file_path):
+                self._logger.info(f'Uploading secret file "{secret_file_path}"...')
+                with open(secret_file_path, 'rb') as secret_file:
+                    self._s3_client.upload_fileobj(
+                        secret_file,
+                        self._aws_backend_bucket,
+                        f'{self._aws_backend_folder}/{secret_file_name}'
+                    )
+
     def update_repos(self):
         """
         Updates the code in the instance repos.
